import java.util.ArrayList;
import java.util.HashSet;
import java.util.Random;

/**
 *
 * @author Joost
 */
public class Specimen implements Comparable<Specimen> {
	
	private final ArrayList<Integer> attributes;
	private final ArrayList<Integer> examples;
	private final ArrayList<Sample> samples;
	private final ArrayList<Boolean> errorvector;
	private double fitness;
	private int correct;

	public Specimen (ArrayList<Sample> samples) {
		this(samples, new HashSet<Integer>());
	}
	
	public Specimen(ArrayList<Sample> samples, HashSet<Integer> filteredAttr) {
		this.examples = new ArrayList<Integer>();
		Random r = new Random();
		for (int i = 0; i < samples.size(); i++) {
			if (r.nextBoolean()) {
				examples.add(i);
			}
		}
		this.attributes = new ArrayList<Integer>();
		for (int i = 0; i < samples.get(0).getAttributeCount(); i++) {
			if (r.nextBoolean() && !filteredAttr.contains(i)) {
				attributes.add(i);
			}
		}
		this.samples = samples;
		this.errorvector = new ArrayList<Boolean>();
		this.correct = findCorrectCount();
	}
	
	public Specimen (ArrayList<Integer> attributes, ArrayList<Integer> examples, ArrayList<Sample> samples) {
		this.attributes = attributes;
		this.examples = examples;
		this.samples = samples;
		this.errorvector = new ArrayList<Boolean>();
		this.correct = findCorrectCount();
	}

	private int findCorrectCount() {
		int correct = 0;
		for (Sample sample:this.samples) {
			int c = computeClassification(sample);
			if (sample.getClassification() == c) {
				correct++;
				errorvector.add(false);
			}
			else {
				errorvector.add(true);
			}
		}
		return correct;
	}
	
	public int computeClassification(ArrayList<Double> att) {
		return computeClassification(new Sample(att, 0, 0));
	}
	
	/**
	 * Deze functie is enorm versimpelt, aangezien 't eerst een generalisatie voor K = n was
	 * maar ons algoritme eigenlijk alleen naar K = 1 kijkt. En dan is 't gewoon de dichstbijzijnde
	 */
	public int computeClassification(Sample sample) {
		Sample minsample = null;
		double minDistSq = Double.MAX_VALUE; 
		for (int example:examples) {
			Sample t = samples.get(example);
			double d = sample.distanceSquared(t, attributes);
			if (d < minDistSq) {
				minDistSq = d;
				minsample = t;
			}
		}
		if (minsample != null)
			return minsample.getClassification();
		return 0;
	}
	
	public ArrayList<Integer> getExamples() {
		return examples;
	}
	
	public ArrayList<Integer> getAttributes() {
		return attributes;
	}
	
	public ArrayList<Sample> getSamples() {
		return samples;
	}
	
	public int getExamplesCount() {
		return examples.size();
	}
	
	public int getAttributeCount() {
		return attributes.size();
	}
	
	public int getCorrectSamples() {
		return correct;
	}
	
	public int getIncorrectSamples() {
		return samples.size() - correct;
	}

	public int getSampleAttributeCount() {
		return samples.get(0).getAttributeCount();
	}

	public int getSampleCount() {
		return samples.size();
	}
	
	@Override
	public String toString() {
		return "Specimen details:\nAttributes used ["+attributes.size()+"]: "+attributes+"\nexamples used ["+examples.size()+"]: "+examples+"\nCorrectly classified: "+correct;
	}

	public int errorDist(Specimen s) {
		int d = 0;
		for (int i = 0; i < errorvector.size(); i++) {
			if (errorvector.get(i) == true && s.errorvector.get(i) == false) {
				d++;
			}
		}
		return d;
	}
	
	public void setFitness(double fitness) {
		this.fitness = fitness;
	}

	public int compareTo(Specimen arg0) {
		return (arg0.fitness - this.fitness < 0 ? -1 : 1);
	}
}
